<?php
/**
 * Plugin Name: Supr3.me Short Linker
 * Plugin URI:  http://www.supr3.me/plugin/supr3me-short-linker/
 * Description: Supr3.me short linker to generate short links. Requires API key for the Supr3.me shortlinker service
 * Version:     1.1.0
 * Author:      Supr3.me Plugins
 * Author URI:  http://www.supr3.me
 *
 * @author Supr3me Plugins <tech@supr3.me>
 */

// load in the Supr3meBaseAbstract class
if (!class_exists('Supr3meBaseAbstract')) {
    require(__DIR__ . '/lib/Supr3meBaseAbstract.php');
}

class Supr3meShortLinker extends Supr3meBaseAbstract
{

    protected
        /**
         * Store our cURL data so we can use it
         *
         * @var array
         */
        $_cURLData = array();

    /**
     * Plugin constructor class
     */
    public function __construct()
    {

        // build our shortlinker
        parent::__construct('short-linker', array(
            'apiurl' => null,
            'apikey' => null,
        ));

        // run the following actions
        $this->action('wp_dashboard_setup', 'action_wp_dashboard_setup');

        // ajax actions
        $this->action('wp_ajax_supr3me-shortlinker-settings-save', 'ajax_settings_save');
        $this->action('wp_ajax_supr3me-shortlinker-generate', 'ajax_shortlinker_generate');

    }


    /**
     * Run this to render our plugin
     *
     * @return void
     */
    public function action_wp_dashboard_setup()
    {

        // add our dashboard widget
        wp_add_dashboard_widget(
            'supr3me-short-linker',
            'Supr3.me Short Linker' . (current_user_can('manage_options') ? ' <a href="#settings" style="float: right; font-size: 10px; margin-right: 15px">Settings</a>' : null),
            array($this, 'load_widget_view')
        );

    }


    /**
     * Load our view for our dashboard widget
     *
     * @return void
     */
    public function load_widget_view()
    {
        $this->load_view('dashboard-widget');
    }


    /**
     * Save our short linker settings
     *
     * @return void
     */
    public function ajax_settings_save()
    {

        // verify our nonce
        if (!wp_verify_nonce($_POST['__nonce'], 'save_shortlinker_settings')) {
            exit('Invalid nonce, please reload page.');
        }

        // check if user can manage options
        if (!current_user_can('manage_options')) {
            exit('You do not have the capability to edit settings.');
        }

        // update our options
        $this->update_option('apiurl', $_POST['api_url']);
        $this->update_option('apikey', $_POST['api_key']);

        // settings have been updated
        exit('Settings have been updated');
    }


    /**
     * Generate shortlink URLs
     *
     * @return void
     */
    public function ajax_shortlinker_generate()
    {

        // verify our nonce
        if (!wp_verify_nonce($_POST['__nonce'], 'send_shortlinker_form')) {
            exit('Invalid nonce, please reload page.');
        }

        // generate our link based on the data posted
        $response = $this->generate($_POST['shortlink_url']);

        // return our response
        header('Content-Type: application/json');
        exit(json_encode($response));

    }

    /**
     * Private method to connect with our API and generate a shortlink
     *
     * @param  string $long_url The long URL that should be converted
     *
     * @return stdClass         Resulting data array
     */
    public function generate($long_url)
    {

        // begin our cURL request
        $ch = curl_init();

        $payload = ['url' => $long_url];

        // build our options
        $options = array(
            CURLOPT_URL => rtrim($this->_options['apiurl'], '/')
                . '/api/create',

            CURLOPT_HTTPHEADER => [
                'Authorization: Bearer ' . $this->_options['apikey'],
                'Content-Type: application/json',
                'Accept: application/json',
            ],

            CURLOPT_POSTFIELDS => json_encode($payload),

            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HEADER => false,
            CURLOPT_POST => true,
        );

        // set our options
        curl_setopt_array($ch, $options);

        // get our response details
        $this->_cURLData['result'] = curl_exec($ch);
        $this->_cURLData['info'] = curl_getinfo($ch);

        // build up our response
        $response = new StdClass();
        $response->success = false;
        $response->data = @json_decode($this->_cURLData['result']);
        $response->message = null;

        // check the response that we got from the API
        switch ($this->_cURLData['info']['http_code']) {

            case 201:
                // shortlink created
                $response->success = true;
                break;

            case 0:
                $response->message = 'Failed to connect to API, please verify your settings.';
                break;

            default:
                $response->message = $response->data->message;
                break;
        }

        // return our response
        return $this->_cURLData['response'] = $response;
    }
}

// instantiate our class
$oSupr3meShortLinker = new Supr3meShortLinker();

// create our global shortlink function
if (!function_exists('shortlink_create')) {

    /**
     * Method to quickly generate a shortlink anywhere in WordPress
     *
     * @param  string $url The URL to convert to a short URL
     *
     * @return bool|string      Returns false or the newly generated URL
     */
    function shortlink_create($url)
    {

        // get globals
        global $oSupr3meShortLinker;

        // get our response
        $response = $oSupr3meShortLinker->generate($url);

        // make sure we returned an object
        if (!$response instanceof StdClass) {
            return false;
        }

        // return result
        return $response->success ? $response->data->data->id : $response->success;
    }
}